"""
Written by Matteo Dunnhofer - 2019

Data provider class
"""
import sys
sys.path.insert(0, '..')

import random
import os
import argparse
import copy
import numpy as np
from got10k.datasets import OTB, VOT, GOT10k, LaSOT, ImageNetVID, UAV123
from PIL import Image
from torch.utils.data.dataset import Dataset
import utils as ut
from demotrackers.DemoTracker import DemoTracker

class DataProvider(Dataset):
    """
    Defines an object that is responsible to load sequences
    from a tracking dataset
    """

    #def __init__(self, cfg, mode, transform=None):
    def __init__(self, cfg, mode, dataset):
        """

        Args:
            mode: string, 'train' or 'validation'
        """
        self.cfg = cfg
        self.mode = mode
        self.dset = dataset
        self.remove_vot_seqs = self.cfg.REMOVE_VOT_SEQS
        self.USE_SHORT_SEQS = self.cfg.USE_SHORT_SEQS if 'train' in self.mode else False

        #random.seed(self.cfg.SEED)+
        #np.random.seed(self.cfg.SEED)

        self.get_meta = False

        if dataset == 'GOT-10k':
            GOT_DIR = os.path.join(self.cfg.DATA_PATH, 'GOT-10k')
            if ('train' in self.mode): #or ('create' in self.mode):
                self.dataset = GOT10k(root_dir=GOT_DIR, subset='train', return_meta=self.get_meta)
                self.starting_idx = 0
                self.ending_idx = len(self.dataset)-1 #9300# 1000 #144 #500
            else:
                self.dataset = GOT10k(root_dir=GOT_DIR, subset='val')
                self.starting_idx = 0
                self.ending_idx = len(self.dataset)-1
        elif dataset == 'OTB2013':
            OTB_DIR = os.path.join(self.cfg.DATA_PATH, 'OTB')
            self.dataset = OTB(root_dir=OTB_DIR, version=2013)
            self.starting_idx = 0
            self.ending_idx = len(self.dataset)-1
        elif dataset == 'OTB2015':
            OTB_DIR = os.path.join(self.cfg.DATA_PATH, 'OTB')
            self.dataset = OTB(root_dir=OTB_DIR, version=2015)
        elif dataset == 'LaSOT':
            LASOT_DIR = os.path.join(self.cfg.DATA_PATH, 'LaSOTBenchmark')
            self.dataset = LaSOT(root_dir=LASOT_DIR, subset='train')
            self.starting_idx = 0
            self.ending_idx = len(self.dataset)-1
        elif dataset == 'UAV123':
            UAV_DIR = os.path.join(self.cfg.DATA_PATH, 'UAV123')
            self.dataset = UAV123(root_dir=UAV_DIR,)
            self.starting_idx = 0
            self.ending_idx = len(self.dataset)-1
        elif dataset == 'ImageNetVID':
            VID_DIR = os.path.join(self.cfg.DATA_PATH, 'ILSVRC2015')
            self.dataset = ImageNetVID(root_dir=VID_DIR, subset='train')
            self.starting_idx = 0
            self.ending_idx = len(self.dataset)-1
        else:
            VOT_DIR = os.path.join(self.cfg.DATA_PATH, 'VOT')
            self.dataset = VOT(root_dir=VOT_DIR, version=2014)
            self.starting_idx = 0
            self.ending_idx = len(self.dataset) - 1

        self.sequence_idx = 0

        self.sequence_idxs = list(range(self.starting_idx, self.ending_idx))
        self.norm_seq_idxs = list(range(len(self.sequence_idxs)))

        #self.sequence_idxs, self.norm_seq_idxs = self.get_very_good_sequences()

        self.demo_trackers = []
        for t_n in self.cfg.TRACKERS:
            self.demo_trackers.append(DemoTracker(self.cfg, t_n, self.dset))

        f = open(os.path.join(self.cfg.EXPERIMENT_PATH, 'n_sequences.txt'), 'w')

        if 'train' in self.mode:
            if self.cfg.DEMO_FILTER == 'PER_FRAME':
                self.sequence_idxs, self.norm_seq_idxs, self.demo_trackers_names, avg_iou, prec = self.get_very_good_sequences_per_frame()
            elif self.cfg.DEMO_FILTER == 'PER_SEQUENCE':
                self.sequence_idxs, self.norm_seq_idxs, self.demo_trackers_names, avg_iou, prec = self.get_very_good_sequences_per_sequence()
            elif self.cfg.DEMO_FILTER == 'FUSION':
                self.sequence_idxs, self.norm_seq_idxs, self.demo_trackers_names, avg_iou, prec = self.get_very_good_sequences_per_frame_fusion()


            if self.dset == 'GOT-10k' and self.remove_vot_seqs:
                self.sequence_idxs, self.norm_seq_idxs = self.remove_vot_banned_sequences()

            if self.cfg.SAMPLE_LIMIT != -1:
                self.sequence_idxs = self.sequence_idxs[:self.cfg.SAMPLE_LIMIT] ##random.sample(self.sequence_idxs, self.cfg.SAMPLE_LIMIT)
                self.norm_seq_idxs = list(range(len(self.sequence_idxs)))

            print('Number of usable sequences: {}'.format(len(self.sequence_idxs)))
            print('Avg IOU: {}'.format(avg_iou))
            print('Precision: {}'.format(prec))
            f.write('Number of usable sequences: {} (i.e. avg IOU > 0.5) \n'.format(len(self.sequence_idxs)))
            f.write('Avg IOU: {}'.format(avg_iou))
            f.write('Precision: {}'.format(prec))
            print('FULL SEQUENCES')
            f.write('FULL SEQUENCES')
            for tr in self.cfg.TRACKERS:
                tr_count = sum(map(lambda x : x == tr, self.demo_trackers_names))
                print('Tracker {} presence: {} ({})'.format(tr, tr_count, (float(tr_count) / len(self.demo_trackers_names)) * 100 ))
                f.write('Tracker {} presence: {} ({})'.format(tr, tr_count, (float(tr_count) / len(self.demo_trackers_names)) * 100 ))



            #self.sequence_idxs = [429, 123, 322, 23]
            #if self.cfg.NUM_SEQS > 0:
            #self.sequence_idxs = self.sequence_idxs[:self.cfg.NUM_SEQS]
            #self.norm_seq_idxs = self.norm_seq_idxs[:self.cfg.NUM_SEQS]

            if self.USE_SHORT_SEQS:
                self.sequence_short_names, self.norm_seq_idxs, self.demo_trackers_names = self.load_short_sequences()
            else:
                self.sequence_idxs = [self.sequence_idxs[i] for i in self.cfg.SEQS_IDXS]
                self.demo_trackers_names = [self.demo_trackers_names[i] for i in self.cfg.SEQS_IDXS]
                self.norm_seq_idxs = list(range(len(self.sequence_idxs)))


        else:
            self.demo_trackers_names = self.get_demo_tracker_names()


        self.n_sequences = len(self.norm_seq_idxs)
        print('Number of total sequences: {}'.format(self.n_sequences))
        f.write('Number of total sequences: {} (i.e. total training sequences)'.format(self.n_sequences))
        print('SEQUENCES USED FOR TRAINING')
        f.write('SEQUENCES USED FOR TRAINING')

        f.close()

        if 'train' in self.mode:
            random.shuffle(self.norm_seq_idxs)


    def get_next_sequence(self, end_idx=-1):
        if self.USE_SHORT_SEQS:
            return self.get_next_short_sequence(end_idx=end_idx)
        else:
            return self.get_next_sequence_normal(end_idx=end_idx)

                
    def get_next_sequence_normal(self, end_idx=-1):
        """
        Loads the next tracking sequence
        """

        norm_idx = self.norm_seq_idxs[self.sequence_idx]
        sequence_idx = self.sequence_idxs[norm_idx]
        seq_idx = copy.deepcopy(norm_idx)

        frames_names, annotations = self.dataset[sequence_idx]

        rand_f_idx = 0

        if 'train' in self.mode:
            rand_f_idx = 0#random.randint(0, len(frames_names) - self.cfg.SEQ_LENGTH - 1) if len(frames_names) > self.cfg.SEQ_LENGTH else 0
            #frames_names, annotations = frames_names[rand_idx:rand_idx+self.cfg.SEQ_LENGTH], annotations[rand_idx:rand_idx+self.cfg.SEQ_LENGTH]
            #print(end_idx)
            frames_names, annotations = frames_names[rand_f_idx:end_idx], annotations[rand_f_idx:end_idx]
        
        frames = []
        for fn in frames_names:
            frames.append(Image.open(fn).convert('RGB'))

        sequence_shape = [len(frames)] + list(frames[0].size)
        sequence_name = self.dataset.seq_names[sequence_idx]
        #print(sequence_name, seq_idx)

        demo_tracker_name = self.demo_trackers_names[norm_idx]
        for i, tr in enumerate(self.demo_trackers):
            if tr.name == demo_tracker_name:
                demo_tracker_idx = i
        demo_annotations = self.demo_trackers[demo_tracker_idx].load_predictions(sequence_name)

        self.sequence_idx = (self.sequence_idx + 1) % self.n_sequences

        if ('train' in self.mode) and (self.sequence_idx == 0):
            random.shuffle(self.norm_seq_idxs)

        return frames, annotations, demo_annotations, sequence_name, sequence_shape, rand_f_idx, seq_idx, demo_tracker_name

    def get_next_short_sequence(self, end_idx=-1):
        """
        Loads the next short tracking sequence
        """

        norm_idx = self.norm_seq_idxs[self.sequence_idx]
        sequence_short_name = self.sequence_short_names[norm_idx]
        sequence_name, start_idx = sequence_short_name.split('.')[0], int(sequence_short_name.split('.')[1])

        seq_idx = copy.deepcopy(norm_idx)

        frames_names, annotations = self.dataset[sequence_name]

        rand_f_idx = start_idx

        frames_names, annotations = frames_names[rand_f_idx:rand_f_idx+self.cfg.SEQ_LENGTH], annotations[rand_f_idx:rand_f_idx+self.cfg.SEQ_LENGTH]
        frames_names, annotations = frames_names[0:end_idx], annotations[0:end_idx]

        frames = []
        for fn in frames_names:
            frames.append(Image.open(fn).convert('RGB'))

        sequence_shape = [len(frames)] + list(frames[0].size)
        # print(sequence_name, seq_idx)
        """
        demo_tracker_name = self.demo_trackers_names[norm_idx]
        for i, tr in enumerate(self.demo_trackers):
            if tr.name == demo_tracker_name:
                demo_tracker_idx = i
        demo_annotations = self.demo_trackers[demo_tracker_idx].load_predictions(sequence_name)
        demo_annotations = demo_annotations[rand_f_idx:rand_f_idx+self.cfg.SEQ_LENGTH]
        demo_annotations = demo_annotations[0:end_idx]
        """
        if self.cfg.DEMO_FILTER == 'FUSION':
            demo_tracker_names = self.demo_trackers_names[norm_idx]
            demo_tracker_name = 'FUSION'
            predictions = [[], [], [], []]
            for i, tr in enumerate(self.demo_trackers):
                for dmtn in demo_tracker_names:
                    if tr.name == dmtn:
                        pred = self.demo_trackers[i].load_predictions(sequence_name)
                        pred = pred[rand_f_idx:rand_f_idx + self.cfg.SEQ_LENGTH]
                        pred = pred[0:end_idx]
                        predictions[i] += pred

            demo_annotations = []
            for i in range(len(annotations)):
                frame_ious = np.zeros(len(self.demo_trackers))
                for j, pred in enumerate(predictions):
                    if len(pred) > 0:
                        frame_ious[j] = ut.iou(pred[i], annotations[i])

                max_pred = np.argmax(frame_ious)
                demo_annotations.append(predictions[max_pred][i])
        else:
            demo_tracker_name = self.demo_trackers_names[norm_idx]
            for i, tr in enumerate(self.demo_trackers):
                if tr.name == demo_tracker_name:
                    demo_tracker_idx = i
            demo_annotations = self.demo_trackers[demo_tracker_idx].load_predictions(sequence_name)
            demo_annotations = demo_annotations[rand_f_idx:rand_f_idx + self.cfg.SEQ_LENGTH]
            demo_annotations = demo_annotations[0:end_idx]

        self.sequence_idx = (self.sequence_idx + 1) % self.n_sequences

        if ('train' in self.mode) and (self.sequence_idx == 0):
            random.shuffle(self.norm_seq_idxs)

        return frames, annotations, demo_annotations, sequence_name, sequence_shape, rand_f_idx, seq_idx, demo_tracker_name


    def get_next_sequence_idx(self):
        """
        Loads the next tracking sequence
        """
        norm_idx = self.norm_seq_idxs[self.sequence_idx]
        seq_idx = copy.deepcopy(norm_idx)

        return seq_idx


    def get_very_good_sequences_per_frame(self):
        """
        Compute the indexes of the sequences which frames iou is above some threshold
        """
        #COEFF = 0.5

        good_idxs = []
        demo_tracker_names = []

        best_ious = []
        best_precs = []

        for idx in self.sequence_idxs:

            if not self.get_meta:
                _, annotations = self.dataset[idx]
            else:
                _, annotations, meta = self.dataset[idx]
                #print(meta['major_class'])

            sequence_name = self.dataset.seq_names[idx]

            # check tracker predictions which are above the threshold
            demo_tracker_ious = np.zeros(len(self.demo_trackers))
            total_ious = [[], [], [], []]
            total_precs = [[], [], [], []]

            for ti, tr in enumerate(self.demo_trackers):
                tr.load_predictions(sequence_name)

                ious = []
                precs = []

                good_tr = True
                for i, a in enumerate(annotations[1:]):
                    iou = ut.iou(a, tr.get_pred(i + 1))
                    ious.append(iou)

                    dist = ut.distance(a, tr.get_pred(i + 1))
                    if dist > 20:
                        prec = 0.0
                    else:
                        prec = 1.0
                    precs.append(prec)

                    if iou < self.cfg.DEMO_GOOD_COEFF:
                        good_tr = False
                        break

                if good_tr:
                    demo_tracker_ious[ti] = np.array(ious).mean() # np.median(np.array(ious))
                    total_ious[ti] += ious
                    total_precs[ti] += precs
                    #total_precs[ti].append(np.array(precs).mean())


            # if one (or more) of the elements of  this vector is > 0 it means that the tracker has good predictions
            if np.count_nonzero(demo_tracker_ious) > 0:
                good_idxs.append(idx)

                # if one or more trackers are good, choose th best performing based on average iou on the sequence
                best_tracker_idx = np.argmax(demo_tracker_ious)

                demo_tracker_names.append(self.demo_trackers[best_tracker_idx].name)

                best_ious += total_ious[best_tracker_idx]
                best_precs += total_precs[best_tracker_idx]

            """
            #print(sequence_name + '.txt',
            #    os.path.exists(os.path.join(self.cfg.DATA_PATH, 'results', self.dset, trackers[0].name, sequence_name)))
            #if os.path.isfile(os.path.join(self.cfg.DATA_PATH, 'results', self.dset, trackers[0].name, sequence_name + '.txt')):
            #for t in trackers:
            trackers[0].load_predictions(sequence_name)

            ious = np.zeros(len(trackers))

            #trs = copy.deepcopy(trackers)
            good = True
            for i, a in enumerate(annotations[1:]):
                if ut.iou(a, trackers[0].get_pred(i+1)) < COEFF:
                    good = False

            if good:
                good_idxs.append(idx)
            """

        #print(len(good_idxs))
        #print(len(best_ious))
        #print(len(best_precs))

        norm_idxs = list(range(len(good_idxs)))
        
        return good_idxs, norm_idxs, demo_tracker_names, np.array(best_ious).mean(), np.array(best_precs).mean()

    def get_very_good_sequences_per_sequence(self):
        """
        Compute the indexes of the sequences which overall iou is above some threshold
        """
        #COEFF = 0.5

        good_idxs = []
        demo_tracker_names = []

        best_ious = []
        best_precs = []

        for idx in self.sequence_idxs:

            if not self.get_meta:
                _, annotations = self.dataset[idx]
            else:
                _, annotations, meta = self.dataset[idx]
                # print(meta['major_class'])

            sequence_name = self.dataset.seq_names[idx]
            #print(sequence_name)

            # check tracker predictions which are above the threshold
            demo_tracker_ious = np.zeros(len(self.demo_trackers))
            total_ious = [[], [], [], []]
            total_precs = [[], [], [], []]

            for ti, tr in enumerate(self.demo_trackers):
                tr.load_predictions(sequence_name)

                ious = []
                precs = []

                good_tr = True
                for i, a in enumerate(annotations[1:]):
                    iou = ut.iou(a, tr.get_pred(i + 1))
                    ious.append(iou)

                    dist = ut.distance(a, tr.get_pred(i + 1))
                    if dist > 20:
                        prec = 0.0
                    else:
                        prec = 1.0
                    precs.append(prec)

                    #if iou < COEFF:
                    #    good_tr = False
                    #    break

                mean_iou = np.array(ious).mean() #np.median(np.array(ious)) #np.array(ious).mean()

                if mean_iou >= self.cfg.DEMO_GOOD_COEFF:
                    demo_tracker_ious[ti] = mean_iou
                    total_ious[ti] += ious
                    total_precs[ti] += precs
                    # total_precs[ti].append(np.array(precs).mean())

            # if one (or more) of the elements of  this vector is > 0 it means that the tracker has good predictions
            if np.count_nonzero(demo_tracker_ious) > 0:
                good_idxs.append(idx)

                # if one or more trackers are good, choose th best performing based on average iou on the sequence
                best_tracker_idx = np.argmax(demo_tracker_ious)

                demo_tracker_names.append(self.demo_trackers[best_tracker_idx].name)

                best_ious += total_ious[best_tracker_idx]
                best_precs += total_precs[best_tracker_idx]


        # print(len(good_idxs))
        # print(len(best_ious))
        # print(len(best_precs))

        norm_idxs = list(range(len(good_idxs)))

        return good_idxs, norm_idxs, demo_tracker_names, np.array(best_ious).mean(), np.array(best_precs).mean()

    def get_very_good_sequences_per_frame_fusion(self):
        """
        Compute the indexes of the sequences which frames iou is above some threshold
        """
        # COEFF = 0.5

        good_idxs = []
        demo_tracker_names = []

        best_ious = []
        best_precs = []

        for idx in self.sequence_idxs:

            if not self.get_meta:
                _, annotations = self.dataset[idx]
            else:
                _, annotations, meta = self.dataset[idx]
                # print(meta['major_class'])

            sequence_name = self.dataset.seq_names[idx]

            # check tracker predictions which are above the threshold
            demo_tracker_ious = np.zeros(len(self.demo_trackers))
            total_ious = [[], [], [], []]
            total_precs = [[], [], [], []]

            predictions = []

            for ti, tr in enumerate(self.demo_trackers):
                pred = tr.load_predictions(sequence_name)

                ious = []
                precs = []

                good_tr = True
                for i, a in enumerate(annotations[1:]):
                    iou = ut.iou(a, tr.get_pred(i + 1))
                    ious.append(iou)

                    dist = ut.distance(a, tr.get_pred(i + 1))
                    if dist > 20:
                        prec = 0.0
                    else:
                        prec = 1.0
                    precs.append(prec)

                    if iou < self.cfg.DEMO_GOOD_COEFF:
                        good_tr = False
                        break

                if good_tr:
                    demo_tracker_ious[ti] = np.array(ious).mean()
                    total_ious[ti] += ious
                    total_precs[ti] += precs
                    # total_precs[ti].append(np.array(precs).mean())

                    predictions.append(pred)


            # if one (or more) of the elements of  this vector is > 0 it means that the tracker has good predictions
            if np.count_nonzero(demo_tracker_ious) > 0:
                good_idxs.append(idx)

                # if one or more trackers are good, choose th best performing based on average iou on the sequence
                best_tracker_names = []
                for ii, diou in enumerate(demo_tracker_ious):
                    if diou > 0:
                        best_tracker_names.append(self.demo_trackers[ii].name)

                demo_tracker_names.append(best_tracker_names)

                demo_fused_iou = []
                demo_fused_prec = []
                for ii in range(len(annotations)):
                    frame_ious = []
                    for pred in predictions:
                        frame_ious.append(ut.iou(pred[ii], annotations[ii]))

                    max_pred = np.argmax(frame_ious)
                    max_iou = ut.iou(predictions[max_pred][ii], annotations[ii])
                    demo_fused_iou.append(max_iou)

                    dist = ut.distance(annotations[ii], predictions[max_pred][ii])
                    if dist > 20:
                        prec = 0.0
                    else:
                        prec = 1.0
                    demo_fused_prec.append(prec)

                best_ious += demo_fused_iou
                best_precs += demo_fused_prec

        # print(len(good_idxs))
        # print(len(best_ious))
        # print(len(best_precs))

        norm_idxs = list(range(len(good_idxs)))

        return good_idxs, norm_idxs, demo_tracker_names, np.array(best_ious).mean(), np.array(best_precs).mean()

    def remove_vot_banned_sequences(self):
        """
        Compute statistics of the dataset
        :return:
        """
        with open(os.path.join(self.cfg.DATA_PATH, 'vot-got10k-banned-seq.txt'), 'r') as f:
            ls = f.readlines()

            banned_seqs_names = [ l.strip('\n') for l in ls]
            #print(banned_seqs_names)

            good_idxs = []
            for idx in self.sequence_idxs:

                sequence_name = self.dataset.seq_names[idx]

                if not sequence_name in banned_seqs_names:
                    good_idxs.append(idx)

        norm_idxs = list(range(len(good_idxs)))

        return good_idxs, norm_idxs

    def get_demo_tracker_names(self):
        demo_tracker_names = []

        for idx in self.sequence_idxs:

            if not self.get_meta:
                _, annotations = self.dataset[idx]
            else:
                _, annotations, meta = self.dataset[idx]
                # print(meta['major_class'])

            sequence_name = self.dataset.seq_names[idx]

            # check tracker predictions which are above the threshold
            demo_tracker_ious = np.zeros(len(self.demo_trackers))

            for ti, tr in enumerate(self.demo_trackers):
                tr.load_predictions(sequence_name)

                ious = []

                for i, a in enumerate(annotations[1:]):
                    iou = ut.iou(a, tr.get_pred(i + 1))
                    ious.append(iou)

                demo_tracker_ious[ti] = np.array(ious).mean()

            best_tracker_idx = np.argmax(demo_tracker_ious)
            demo_tracker_names.append(self.demo_trackers[best_tracker_idx].name)

        return demo_tracker_names

    def create_shorter_sequences(self):
        """
        Creates a file where each line is: seq_name.start_idx
        to have a fixed bigger dataset of sequences that are shorter
        :return:
        """
        range_w = 4
        def good(r, already_chosen):

            good = True

            if len(already_chosen) > 0:
                if r in already_chosen:
                    good = False

                for ac in already_chosen:
                    if r in list(range(ac - range_w, ac + range_w)):
                        good = False

            return good

        random.seed(self.cfg.SEED)
        np.random.seed(self.cfg.SEED)

        IDXS_PER_SEQ = 5 #self.cfg.SHORT_SEQS_PER_SEQ
        with open(self.dset + '_short_train_seqs-FULL.txt', 'w') as f:

            for idx in self.sequence_idxs:

                if not self.get_meta:
                    _, annotations = self.dataset[idx]
                else:
                    _, annotations, meta = self.dataset[idx]
                    # print(meta['major_class'])

                sequence_name = self.dataset.seq_names[idx]
                print(sequence_name, len(annotations))

                chosen_idxs = []

                if len(annotations) > self.cfg.SEQ_LENGTH:
                    idxs_per_seq = ut.clip(int((len(annotations) - self.cfg.SEQ_LENGTH) / (2 * range_w)), 0, IDXS_PER_SEQ)
                    print(idxs_per_seq)
                    for _ in range(idxs_per_seq):
                        r_idx = random.randint(0, len(annotations)-1 - self.cfg.SEQ_LENGTH)

                        while not good(r_idx, chosen_idxs):
                            r_idx = random.randint(0, len(annotations) - 1 - self.cfg.SEQ_LENGTH)

                        chosen_idxs.append(r_idx)

                        f.write(sequence_name + '.' + str(r_idx) + '\n')
                else:
                    f.write(sequence_name + '.' + str(0) + '\n')

            f.close()

    def load_short_sequences(self):
        """
        Load short sequences from file
        """
        short_sequences_names = []
        demo_tracker_names = []
        with open('/media/TBData2/projects/vot-rl-demo/src/data/' + self.dset + '_short_train_seqs-FULL.txt', 'r') as f:

            lines = f.readlines()

            for line in lines:
                line = line.rstrip()
                sequence_name = line.split('.')[0]
                sequence_idx = self.dataset.seq_names.index(sequence_name)

                # if the sequence is good enough, i.e. it has a demonstrator with good enough performance
                if sequence_idx in self.sequence_idxs:
                    short_sequences_names.append(line)

                    best_tracker_name = self.demo_trackers_names[self.sequence_idxs.index(sequence_idx)]
                    demo_tracker_names.append(best_tracker_name)

            f.close()

        return short_sequences_names, list(range(len(short_sequences_names))), demo_tracker_names #['SiamFC'] * len(short_sequences_names)

    # def get_good_sequences_per_frame(self):
    #     """
    #     Compute statistics of the dataset
    #     :return:
    #     """
    #     trackers = []
    #     for t_n in self.cfg.TRACKERS:
    #         trackers.append(Tracker(self.cfg, t_n, self.dset))

    #     trackers_bests = np.zeros(len(trackers))

    #     good_idxs = []
    #     per_tracker_good_idxs = [[] for _ in range(len(trackers))]
    #     for idx in self.sequence_idxs:
    #         _, annotations = self.dataset[idx]

    #         sequence_name = self.dataset.seq_names[idx]

    #         for t in trackers:
    #             t.load_predictions(sequence_name)

    #         ious = []

    #         for i, a in enumerate(annotations[1:]):
    #             ious_l = []
    #             for t in trackers:
    #                 ious_l.append(ut.iou(a, t.get_pred(i+1)))

    #             ious.append(ious_l)

    #         success = np.zeros(len(trackers))

    #         for i in range(len(annotations[1:])):
    #             max_idx =  np.argmax(ious[j])
    #             success[max_idx] += 1

    #         success /= len(annotations[1:])
    #         best_success_idx = np.argmax(success)
            
    #         good_trackers = [best_success_idx]
            
    #         GOOD_THRESHOLD = 0.1

    #         for i, s in enumerate(success):
    #             if i != best_success_idx:
    #                 if abs(success[best_success_idx] - s) > GOOD_THRESHOLD:
    #                     break

    #         for i, iou in enumerate(ious):
    #             if iou >= 0.5:
    #                 #print (idx, sequence_name, ious, i)
    #                 good_idxs.append(idx)
    #                 per_tracker_good_idxs[i].append(idx)

    #                 trackers_bests[i] += 1

    #                 break

    #     print(trackers_bests)

    #     min_tracker_samples = int(min(trackers_bests))

    #     trackers_bests /= len(good_idxs)

    #     print(trackers_bests)

    #     final_idxs = []

    #     for i in range(len(trackers)):
    #         final_idxs += random.sample(per_tracker_good_idxs[i], min_tracker_samples)

    #     print(len(final_idxs))
        
    #     return final_idxs

    def compute_seq_mean_std(self, annotations):
        d_x, d_y, d_w, d_h = [], [], [], []

        for i in range(1, len(annotations)):
            new_bb = annotations[i]
            old_bb = annotations[i - 1]

            d_x.append(abs(new_bb[0] - old_bb[0]) / old_bb[2])
            d_y.append(abs(new_bb[1] - old_bb[1]) / old_bb[3])
            d_w.append(abs(new_bb[2] - old_bb[2]) / old_bb[2])
            d_h.append(abs(new_bb[3] - old_bb[3]) / old_bb[3])

        d_x = np.array(d_x)
        d_y = np.array(d_y)
        d_w = np.array(d_w)
        d_h = np.array(d_h)

        return [d_x.mean(), d_y.mean(), d_w.mean(), d_h.mean()], [d_x.std(), d_y.std(), d_w.std(), d_h.std()]


    def shift_stats(self):
        #seq_stds = []
        d_x, d_y, d_w, d_h = [], [], [], []

        for idx in self.sequence_idxs:
            _, annotations = self.dataset[idx]

            sequence_name = self.dataset.seq_names[idx]

            d_x_ps, d_y_ps, d_w_ps, d_h_ps = [], [], [], []

            c = 0
            for i in range(1, len(annotations)):
                c += 1

                new_bb = annotations[i]
                old_bb = annotations[i-1]

                d_x_ps.append(abs(new_bb[0] - old_bb[0]) / old_bb[2])
                d_y_ps.append(abs(new_bb[1] - old_bb[1]) / old_bb[3])
                d_w_ps.append(abs(new_bb[2] - old_bb[2]) / old_bb[2])
                d_h_ps.append(abs(new_bb[3] - old_bb[3]) / old_bb[3])

            d_x += d_x_ps
            d_y += d_y_ps
            d_w += d_w_ps
            d_h += d_h_ps

            d_x_ps = np.array(d_x_ps)
            d_y_ps = np.array(d_y_ps)
            d_w_ps = np.array(d_w_ps)
            d_h_ps = np.array(d_h_ps)

            print(sequence_name)
            print('Shift X: avg {} std {} min {} max {}'.format(d_x_ps.mean(), d_x_ps.std(), d_x_ps.min(), d_x_ps.max()))
            print('Shift Y: avg {} std {} min {} max {}'.format(d_y_ps.mean(), d_y_ps.std(), d_y_ps.min(), d_y_ps.max()))
            print('Shift W: avg {} std {} min {} max {}'.format(d_w_ps.mean(), d_w_ps.std(), d_w_ps.min(), d_w_ps.max()))
            print('Shift H: avg {} std {} min {} max {}'.format(d_h_ps.mean(), d_h_ps.std(), d_h_ps.min(), d_h_ps.max()))
            print('*****************************')

        d_x = np.array(d_x)
        d_y = np.array(d_y)
        d_w = np.array(d_w)
        d_h = np.array(d_h)

        print('FINAL')
        print('Shift X: avg {} std {} min {} max {}'.format(d_x.mean(), d_x.std(), d_x.min(), d_x.max()))
        print('Shift Y: avg {} std {} min {} max {}'.format(d_y.mean(), d_y.std(), d_y.min(), d_y.max()))
        print('Shift W: avg {} std {} min {} max {}'.format(d_w.mean(), d_w.std(), d_w.min(), d_w.max()))
        print('Shift H: avg {} std {} min {} max {}'.format(d_h.mean(), d_h.std(), d_h.min(), d_h.max()))

    def shift_stats_tracker(self):
        tracker =Tracker(self.cfg, 'SiamFC', self.dset)

        for idx in self.sequence_idxs:

            sequence_name = self.dataset.seq_names[idx]

            tracker.load_predictions(sequence_name)

            d_x, d_y, d_w, d_h = [], [], [], []
            c = 0
            for i in range(1, len(tracker.predictions)):
                c += 1

                new_bb = tracker.get_pred(i)
                old_bb = tracker.get_pred(i-1)

                d_x.append(abs(new_bb[0] - old_bb[0]) / old_bb[2])
                d_y.append(abs(new_bb[1] - old_bb[1]) / old_bb[3])
                d_w.append(abs(new_bb[2] - old_bb[2]) / old_bb[2])
                d_h.append(abs(new_bb[3] - old_bb[3]) / old_bb[3])

        d_x = np.array(d_x)
        d_y = np.array(d_y)
        d_w = np.array(d_w)
        d_h = np.array(d_h)

        print('Shift X: avg {} std {} min {} max {}'.format(d_x.mean(), d_x.std(), d_x.min(), d_x.max()))
        print('Shift Y: avg {} std {} min {} max {}'.format(d_y.mean(), d_y.std(), d_y.min(), d_y.max()))
        print('Shift W: avg {} std {} min {} max {}'.format(d_w.mean(), d_w.std(), d_w.min(), d_w.max()))
        print('Shift H: avg {} std {} min {} max {}'.format(d_h.mean(), d_h.std(), d_h.min(), d_h.max()))

    def stats(self):
        """
        Compute statistics of the dataset
        :return:
        """
        n_sequences = len(self.sequence_idxs)

        n_frames = 0
        max_length = 0
        min_length = 1e6
        for idx in self.sequence_idxs:
            _, annotations = self.dataset[idx]

            length = len(annotations)

            n_frames += length

            if length > max_length:
                max_length = length

            if length < min_length:
                min_length = length


        stats_str = 'DATASETS STATS\n\n'

        stats_str += 'Number of videos: {:08d}\n'.format(n_sequences)
        stats_str += 'Number of frames: {:08d}\n'.format(n_frames)
        stats_str += 'Max length: {:08d}\n'.format(max_length)
        stats_str += 'Min length: {:08d}\n'.format(min_length)
        stats_str += 'Avg length: {:.03f}\n'.format(n_frames / n_sequences)

        print(stats_str)


if __name__ == '__main__':
    from config import Configuration

    parser = argparse.ArgumentParser()
    parser.add_argument('--stats', help='Show the dataset statistics', action='store_true')
    parser.add_argument('--shows', help='Draw ground-truth on random sequence', action='store_true')
    args = parser.parse_args()

    config = Configuration()
    config.DEMO_FILTER = 'FUSION'
    config.DEMO_GOOD_COEFF = 0.5 #0.7
    config.TRACKERS = ['ECO', 'SiamRPN', 'KCF', 'MDNet'] #['DiMP', 'ATOM', 'SiamRPN']#['ECO', 'SiamRPN', 'KCF', 'MDNet']
    #dset = DataProvider(config, 'create', 'GOT-10k')
    #dset.create_shorter_sequences()
    #dset = DataProvider(config, 'train', 'GOT-10k')
    dset = DataProvider(config, 'train', 'LaSOT')
    #dset.create_shorter_sequences()


    #dset.stats()
    #dset.get_good_sequences()
    #dset.shift_stats()
    #dset.shift_stats_tracker()



